//+------------------------------------------------------------------+
//|                                      LinearRegressionChannel.mq4 |
//|                                                           dimicr |
//|                                         http://www.signalsbg.com |
//|      SignalsBG.com - development of trading tools for Metatrader |
//////////////////////////////////////////////////////////////////////
#property copyright "Copyright  2008 signalsbg.com"
#property link      "http://www.signalsbg.com"
#property indicator_chart_window

#property indicator_buffers 3
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_color1 Aqua
#property indicator_color2 Green
#property indicator_color3 Yellow

extern bool   UseClose         = true;
extern int    barsToCount      = 200;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsOnHighLow  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = true;
extern bool   alertsNotify     = true;
extern bool   alertsEmail      = true;
extern string soundFile        = "alert2.wav"; 


double LR_line[];
double Sup_line[];
double Res_line[];
double trend[];

//////////////////////////////////////////////////////////////////////
int init()
{
   //IndicatorShortName("LinearRegressionChannel:"+barsToCount);
   IndicatorBuffers(4);
   SetIndexBuffer(0,LR_line);
   SetIndexBuffer(1,Sup_line);
   SetIndexBuffer(2,Res_line);
   SetIndexBuffer(3,trend);
   
   
   return(0);
}

//////////////////////////////////////////////////////////////////////
int deinit()
{  
   return(0);
}

//////////////////////////////////////////////////////////////////////
int start()
{
   // variables
   double a,b,c,
          sumy=0.0,
          sumx=0.0,
          sumxy=0.0,
          sumx2=0.0,
          h=0.0,l=0.0;   
   int x;
   
   // calculate linear regression
   
   for(int i=0; i<barsToCount; i++)
   {
      sumy+=Close[i];
      sumxy+=Close[i]*i;
      sumx+=i;
      sumx2+=i*i;
   }
   
   c=sumx2*barsToCount-sumx*sumx;
   
   if(c==0.0)
   {
      Alert("Error in linear regression!");
      return;
   }
   
   
   LR_line[x]  = EMPTY_VALUE;
   Sup_line[x] = EMPTY_VALUE;
   Res_line[x] = EMPTY_VALUE;   
   b=(sumxy*barsToCount-sumx*sumy)/c;
   a=(sumy-sumx*b)/barsToCount;
   
   // Linear regression line in buffer
   for(x=0;x<barsToCount;x++)
      LR_line[x]=a+b*x;
   
   

   
   // Use PRICE_CLOSE for support-resistance
   if (UseClose)
     for(x=0;x<barsToCount;x++)
     {
       if(Close[x]-LR_line[x] > h) h = Close[x]-LR_line[x];
       if(LR_line[x] - Close[x]> l) l = LR_line[x] - Close[x];
     }  
   
   // Use HIGH - LOW
   else
     for(x=0;x<barsToCount;x++)
     {
       if(High[x]-LR_line[x] > h) h = High[x]-LR_line[x];
       if(LR_line[x] - Low[x]> l) l = LR_line[x] - Low[x];
     }
   
   // Drawing support - resistance lines   
   if (h>l)
   {
     for(x=0;x<barsToCount;x++)
     {
       Sup_line[x]=a-h+b*x;
       Res_line[x]=a+h+b*x;
     } 
   }
   else
   {
     for(x=0;x<barsToCount;x++)
     {
       Sup_line[x]=a-l+b*x;
       Res_line[x]=a+l+b*x;
     }
   }
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
            int forBar = 0;
      else      forBar = 1;
      if (alertsOnHighLow)       
      {
         if (High[forBar]>Res_line[forBar] && High[forBar+1]<Res_line[forBar+1]) doAlert("high penetrated upper bar");
         if ( Low[forBar]<Sup_line[forBar] &&  Low[forBar+1]>Sup_line[forBar+1]) doAlert("low penetrated lower bar");
      }
      else
      {
         if (Close[forBar]>Res_line[forBar] && Close[forBar+1]<Res_line[forBar+1]) doAlert("close penetrated upper bar");
         if (Close[forBar]<Sup_line[forBar] && Close[forBar+1]>Sup_line[forBar+1]) doAlert("close penetrated lower bar");
      }
   }   
   
   

   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="";
   static datetime previousTime;
   string message;

   //
   //
   //
   //
   //
   
   if (previousAlert!=doWhat || previousTime!=Time[0]) 
   {
      previousAlert = doWhat;
      previousTime  = Time[0];

      message= StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," !LinRegrBuf : ",doWhat);
         if (alertsMessage) Alert(message);
         if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," !LinRegrBuf " +" "+message));
         if (alertsEmail)   SendMail(StringConcatenate(Symbol()," !LinRegrBuf  "),message);
         if (alertsSound)   PlaySound(soundFile);
    }
}

